//--------------------------------------------------------------------------
//Tania Morimoto and Allison Okamura, Stanford University
//10.10.13
//Code to test basic Hapkit functionality: position sensing and force output
//--------------------------------------------------------------------------

// Includes
#include <math.h>

// Pin Declares 
int pwmAPin = 5; // for motor
int dirAPin = 8; // for motor
int sensorPosPin = A2; // for MR sensor

// Position tracking variables
int pos = 0;
int actualPos[3] = {0,0,0};  //keep track of 3 positions where 0 index = lastlast, 1 index = last, 2 = current
int flipCount = 0;           //keeps track of the number of flips over the 180deg mark
int tempOffset = 0;
int actualDiff = 0;
int lastactualDiff = 0;
int actualOffset = 0;
int lastactualOffset = 0;
const int flipThresh = 700;
boolean flipped = false;

// Force ourput variables
int forceDir = 0;
double duty = 0;
unsigned int output = 0;

//-----------------------------------
//Initialize
//-----------------------------------
void setup() 
{
  // Set Up Serial
  Serial.begin(9600);
  
  // Set PWM frequency 
  setPwmFrequency(pwmAPin,1); 
  
  // Input Pins
  pinMode(sensorPosPin, INPUT); // mag sensor input pin
  
  // Output Pins
  pinMode(pwmAPin, OUTPUT);  // PWM pin for motor A
  pinMode(dirAPin, OUTPUT);  // dir pin for motor A
  
  // Init Motor 
  analogWrite(pwmAPin, 0);     // set to not be spinning (0/255)
  digitalWrite(dirAPin, LOW);  // set direction
  
  // Init Position
  actualPos[0] = analogRead(sensorPosPin);
  actualPos[1] = analogRead(sensorPosPin);
  
}

//-----------------------------------
// Main Loop
//-----------------------------------
void loop()
{
  // Get voltaage output by MR sensor
  actualPos[2] = analogRead(sensorPosPin);  //current position

  // Print MR sensor voltage to the serial monitor
  Serial.println(actualPos[2]);

  //calculate differences between MR sensor readings
  actualDiff = actualPos[2] - actualPos[1];      //difference btwn current pos and last pos
  lastactualDiff = actualPos[2] - actualPos[0];  //difference btwn current pos and last last pos
  actualOffset = abs(actualDiff);
  lastactualOffset = abs(lastactualDiff);
  
  // Update position record-keeping vairables
  actualPos[0] = actualPos[1];
  actualPos[1] = actualPos[2];
  
  // Keep track of flips over 180 degrees
  if((lastactualOffset > flipThresh) && (!flipped))   //try different values here
  {
    if(lastactualDiff > 0)//(actualDiff > 0)
    {
      flipCount--;   //cw rotation for how I've been playing with MR sensor 
    } 
    else //if(actualDiff<0)
    {
      flipCount++;   //ccw rotation for how I've been playing with MR sensor
    }
    if(actualOffset > flipThresh)
    {
      pos = actualPos[2] + flipCount*actualOffset;      //if data was good and actual offset is above threshold
      tempOffset = actualOffset;
    }else    //in this case there was a blip in the data and we want to use lastactualOffset
    {
      pos = actualPos[2] + flipCount*lastactualOffset;    //update the pos value to account for any flips over 180deg *****changed this so that it is based on the ACTUAL OFFSET!!******
      tempOffset = lastactualOffset;
    }
    flipped = true;    //set boolean so that the next time through the loop won't trigger a flip
  }
  else
  {
    pos = actualPos[2] + flipCount*tempOffset;    //need to update pos based on what most recent offset is 
    flipped = false;
  }

  //--------Force output------------
  forceDir = HIGH; // HIGH is positive, LOW is negative
  duty = 0; // ranges between 0 and 1 (0.5 = 50%)

  // Set the direction of the force
  if(forceDir == HIGH)
  {
    digitalWrite(dirAPin, LOW);
  }
  else
  {
    digitalWrite(dirAPin, HIGH);
  } 
  
  // Set the amount of force using a commanded duty cycle
  if (duty > .80) 
  {
    duty = .80;              // output a maximum of 80% duty cycle
  }    
  if (duty < 0) { 
    duty = 0;
  }  
  output = (int)(duty* 255);   // convert duty cycle to output signal
  analogWrite(pwmAPin,output); // output the signal

}


// ---------------------------------------
// Function to set PWM Freq -- DO NOT EDIT
// ---------------------------------------
void setPwmFrequency(int pin, int divisor) {
  byte mode;
  if(pin == 5 || pin == 6 || pin == 9 || pin == 10) {
    switch(divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 64: mode = 0x03; break;
      case 256: mode = 0x04; break;
      case 1024: mode = 0x05; break;
      default: return;
    }
    if(pin == 5 || pin == 6) {
      TCCR0B = TCCR0B & 0b11111000 | mode;
    } else {
      TCCR1B = TCCR1B & 0b11111000 | mode;
    }
  } else if(pin == 3 || pin == 11) {
    switch(divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 32: mode = 0x03; break;
      case 64: mode = 0x04; break;
      case 128: mode = 0x05; break;
      case 256: mode = 0x06; break;
      case 1024: mode = 0x7; break;
      default: return;
    }
    TCCR2B = TCCR2B & 0b11111000 | mode;
  }
}
